package com.phonegap;

import org.json.JSONArray;
import org.json.JSONException;

import com.phonegap.api.Plugin;
import com.phonegap.api.PluginResult;

import android.content.Intent;
import android.webkit.WebView;

public class CryptoHandler implements Plugin {
	
    WebView webView;					// WebView object
    DroidGap ctx;						// DroidGap object
	
	/**
	 * Constructor.
	 */
	public CryptoHandler() {
	}

	/**
	 * Sets the context of the Command. This can then be used to do things like
	 * get file paths associated with the Activity.
	 * 
	 * @param ctx The context of the main Activity.
	 */
	public void setContext(DroidGap ctx) {
		this.ctx = ctx;
	}

	/**
	 * Sets the main View of the application, this is the WebView within which 
	 * a PhoneGap app runs.
	 * 
	 * @param webView The PhoneGap WebView
	 */
	public void setView(WebView webView) {
		this.webView = webView;
	}

	/**
	 * Executes the request and returns CommandResult.
	 * 
	 * @param action The command to execute.
	 * @param args JSONArry of arguments for the command.
	 * @return A CommandResult object with a status and message.
	 */
	public PluginResult execute(String action, JSONArray args) {
		PluginResult.Status status = PluginResult.Status.OK;
		String result = "";		
		
		try {
			if (action.equals("encrypt")) {
				this.encrypt(args.getString(0), args.getString(1));
			}
			else if (action.equals("decrypt")) {
				this.decrypt(args.getString(0), args.getString(1));
			}
			return new PluginResult(status, result);
		} catch (JSONException e) {
			return new PluginResult(PluginResult.Status.JSON_EXCEPTION);
		}
	}

	/**
	 * Identifies if action to be executed returns a value.
	 * 
	 * @param action	The action to execute
	 * @return			T=returns value
	 */
	public boolean hasReturnValue(String action) {
		return false;
	}

	/**
     * Called when the system is about to start resuming a previous activity. 
     */
    public void onPause() {
    }

    /**
     * Called when the activity will start interacting with the user. 
     */
    public void onResume() {
    }
    
    /**
     * Called by AccelBroker when listener is to be shut down.
     * Stop listener.
     */
    public void onDestroy() { 	
    }

    /**
     * Called when an activity you launched exits, giving you the requestCode you started it with,
     * the resultCode it returned, and any additional data from it. 
     * 
     * @param requestCode		The request code originally supplied to startActivityForResult(), 
     * 							allowing you to identify who this result came from.
     * @param resultCode		The integer result code returned by the child activity through its setResult().
     * @param data				An Intent, which can return result data to the caller (various data can be attached to Intent "extras").
     */
    public void onActivityResult(int requestCode, int resultCode, Intent intent) {
    }

    //--------------------------------------------------------------------------
    // LOCAL METHODS
    //--------------------------------------------------------------------------

	public void encrypt(String pass, String text) {
		try {
			String encrypted = SimpleCrypto.encrypt(pass,text);
			// TODO: Why not just return text now?
			this.ctx.sendJavascript("Crypto.gotCryptedString('" + text + "')");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void decrypt(String pass, String text) {
		try {
			String decrypted = SimpleCrypto.decrypt(pass,text);
			this.ctx.sendJavascript("Crypto.gotPlainString('" + text + "')");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
